function epglmark(lmark)

% EPGLMARK: find specific event ('landmark') in the EPG trace
%
%      EPGLMARK scans the EPG signal from the current cursor rightward
%      looking for a prespecified EPG pattern, and shifts the cursor
%      to that point. Four types of EPG patterns can be identified:
%      a) complete closure in the alveolar region (rows 1-3);
%      b) complete closure in the velar region (rows 6-8);
%      c) release of closure in the alveolar region;
%      d) release of closure in the velar region.
%
%      See also: EPGINDEX, EXTREMUM.

%      N. Nguyen, Oct. 96

global epg epg_sr tlim

if ~exist('epg') | ~exist('epg_sr')

  fprintf(1,'EPG data not available.\n');
  return;

end

t = tscale(epg,epg_sr);

crsr1 = crsrloc(1);
crsr2 = crsrloc(2);

pt = get(gca,'CurrentPoint');

if abs(pt(1,1) - crsr1) < abs(pt(1,1) - crsr2)
  t1 = crsr1;
  crsr = 1;
else
  t1 = crsr2;
  crsr = 2;
end

samp = find(t >= t1 & t <= tlim(2));
nsamp = length(samp);

P = epg(samp(1):samp(nsamp),:);

P = P';

pal = [NaN   1   2   3   4   5   6 NaN;
         7   8   9  10  11  12  13  14;
        15  16  17  18  19  20  21  22;
        23  24  25  26  27  28  29  30;
        31  32  33  34  35  36  37  38;
        39  40  41  42  43  44  45  46;
        47  48  49  50  51  52  53  54;
        55  56  57  58  59  60  61  62];

if findstr(lmark,'ALV')

  P = [sum(P(1:6,:))./6 ; sum(P(7:14,:))./8; sum(P(15:22,:))./8];

elseif findstr(lmark,'VEL')

  P = [sum(P(39:46,:))./8 ; sum(P(47:54,:))./8; sum(P(55:62,:))./8];

end

P = P';

lmark1 = find(P(:,1) == 1 | P(:,2) == 1 | P(:,3) == 1);

if isempty(lmark1)

  fprintf(1,'No complete closure found.\n');
  return;

else

  lmark1 = lmark1(1);

  if findstr(lmark,'CLOS')

    crsrmove(crsr,t(samp(lmark1)) - t1);    

  elseif findstr(lmark,'REL')

    lmark2 = find(P(lmark1:nsamp,1) < 1 & ...
                  P(lmark1:nsamp,2) < 1 & ...
                  P(lmark1:nsamp,3) < 1);

    if isempty(lmark2)

      fprintf(1,'No release found.\n');
      return;

    else

      lmark2 = lmark1 + lmark2(1) - 1;

      crsrmove(crsr,t(samp(lmark2)) - t1);

    end

  end

end

