function [audio,fs,ffmt]=readaud(filename,mode,nmax,nskip)
%READAUD  Read a waveform file [Y,FS,FFX]=(FILENAME,MODE,NMAX,NSKIP)
%
% Input Parameters:
%
%	FILENAME gives the name of the file (with optional .WAV  extension) or alternatively
%                 can be the FFX output from a previous call to READEST having the 'f' mode option
%	MODE		specifies the following (*=default):
%
%    Scaling: 's'    Auto scale to make data peak = +-1 (use with caution if reading in chunks)
%             'r'    Raw unscaled data (integer values)
%             'p' *	Scaled to make +-1 equal full scale
%   Format    'l'    Little endian data (Intel,DEC) (overrides indication in file)
%             'b'    Big endian data (non Intel/DEC) (overrides indication in file)
%   File I/O: 'f'    Do not close file on exit
%
%	NMAX     maximum number of samples to read (or -1 for unlimited [default])
%	NSKIP    number of samples to skip from start of file
%               (or -1 to continue from previous read when FFX is given instead of FILENAME [default])
%
% Output Parameters:
%
%	AUDIO     data matrix of dimension (samples,channels)
%	FS        sample frequency in Hz
%	FFMT      Cell array containing essential data 

%		{1} filenametype [wav,sph,sfs]
%		{2} filename
%		{3} file ID
%		{4} nsamples
%		{5} nchannels
%		{6} nbytes per sample
%		{7} nbits per sample
%		{8} min value
%		{9} max vlaue
%		{10} filetype cell array with other details and used for writing
%
%   If no output parameters are specified, header information will be printed.
%

%	Copyright (C) Alan Wrench 1999
%
%      Last modified Fri Oct 23 07:28:05 1998
%
%   VOICEBOX is a MATLAB toolbox for speech processing. Home page is at
%   http://www.ee.ic.ac.uk/hp/staff/dmb/voicebox/voicebox.html
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You can obtain a copy of the GNU General Public License from
%   ftp://prep.ai.mit.edu/pub/gnu/COPYING-2.0 or by writing to
%   Free Software Foundation, Inc.,675 Mass Ave, Cambridge, MA 02139, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Try reading as microsoft wav file
%       FIDX     Information row vector containing the element listed below.
%
%           (1)  file id
%           (2)  current position in file
%           (3)  dataoff        byte offset in file to start of data
%           (4)  nsamp  number of samples
%           (5)  nchan  number of channels
%           (6)  nbyte  bytes per data value
%           (7)  bits   number of bits of precision
%           (8)  code   Data format: 1=PCM, 2=ADPCM, 6=A-law, 7=Mu-law
%           (9)  fs     sample frequency

 [y,s,wmode,ffx] = readwav(filename, mode, nmax, nskip);
  if ~isempty(y)
	fs = s;
	ffmt{1} = 'wav';
	ffmt{2} = filename;
	ffmt{3} = ffx(1);
	ffmt{4} = ffx(4);
	ffmt{5} = ffx(5);
	ffmt{6} = ffx(6);
	ffmt{7} = ffx(7);
	ffmt{8} = 0;
	ffmt{9} = 0;
	ffmt{10} = ffx;
	audio = y;
	return;
  end
% else try reading as sphere file
%       FFX      Cell array containing
%
%     {1}     filename
%     {2}     header information
%        {1}  first header field name
%        {2}  first header field value
%     {3}     format string (e.g. NIST_1A)
%     {4}(1)  file id
%        (2)  current position in file
%        (3)  dataoff   byte offset in file to start of data
%        (4)  order  byte order (l or b)
%        (5)  nsamp     number of samples
%        (6)  number of channels
%        (7)  nbytes    bytes per data value
%        (8)  bits      number of bits of precision
%        (9)  fs        sample frequency
%        (10) min value
%        (11) max value

 	[y,s,ffx] = readsph(filename, mode, nmax, nskip);
  if ~isempty(y)
	fs = s;
	ffmt{1} = 'sph';
	ffmt{2} = filename;
	ffmt{3} = ffx{4}(1);
	ffmt{4} = ffx{4}(5);
	ffmt{5} = ffx{4}(6);
	ffmt{6} = ffx{4}(7);
	ffmt{7} = ffx{4}(8);
	ffmt{8} = ffx{4}(9);
	ffmt{9} = ffx{4}(10);
	ffmt{10} = ffx;
	audio = y;
	return;
  end
% else try reading as sfs (speech type, first instance)
%  FFX     cell array containing:
%              {1} = filename
%              {2} = (1,4) = [fid byte_order item_row values_read]
%              {3} = (nitem,5) = one row per item [type subtype length position byteorder]
%
	[y,s,hd,ffx]=readsfs(filename,1,1,mode,nmax,nskip)
  if ~isempty(y)
	fs = s;
	ffmt{1} = 'sfs';
	ffmt{2} = filename;
	ffmt{3} = ffx{2}{1};
	ffmt{4} = ffx{1}{8}/ffx{1}{5};
	ffmt{5} = 1;
	ffmt{6} = ffx{1}{5};
	ffmt{7} = 16;
	ffmt{8} = 0;
	ffmt{9} = 0;
	ffmt{10} = ffx;
	audio = y;
	return;
  end
% else read as raw binary with warning
