function [y,s,ffx,t]=readest(filename,mode,nmax,nskip)
%READEST  Read an Edinburgh Speech Tools(EST) format data file [Y,FS,FFX]=(FILENAME,MODE,NMAX,NSKIP)
%
% Input Parameters:
%
%	FILENAME gives the name of the file (with optional .EMA  extension) or alternatively
%                 can be the FFX output from a previous call to READEST having the 'f' mode option
%	MODE		specifies the following (*=default):
%
%    Scaling: 's'    Auto scale to make data peak = +-1 (use with caution if reading in chunks)
%             'r'    Raw unscaled data (integer values)
%             'p' *	Scaled to make +-1 equal full scale
%   Format    'l'    Little endian data (Intel,DEC) (overrides indication in file)
%             'b'    Big endian data (non Intel/DEC) (overrides indication in file)
%   File I/O: 'f'    Do not close file on exit
%
%	NMAX     maximum number of samples to read (or -1 for unlimited [default])
%	NSKIP    number of samples to skip from start of file
%               (or -1 to continue from previous read when FFX is given instead of FILENAME [default])
%
% Output Parameters:
%
%	Y        data matrix of dimension (samples,channels)
%	FS       sample frequency in Hz
%	FFX      Cell array containing
%
%     {1}     filename
%     {2}     header information
%        {1}  first header field name
%        {2}  first header field value
%     {3}     format string (e.g. EST_File)
%     {4}(1)  file id
%        (2)  current position in file (measured in samples)
%        (3)  dataoff	byte offset in file to start of data
%        (4)  order  byte order (l or b)
%        (5)  nsamp	number of samples
%        (6)  number of channels
%        (7)  nbytes	bytes per data value
%        (8)  bits	number of bits of precision
%        (9)  fs	sample frequency
%	 (10) minx value
%        (11) maxx value
%	 (12) miny value
%        (13) maxy value
%     {5}     channel labels
%
%   If no output parameters are specified, header information will be printed.
%

%	Copyright (C) Alan Wrench 1999
%
%      Last modified Fri Oct 23 07:28:05 1998
%
%   VOICEBOX is a MATLAB toolbox for speech processing. Home page is at
%   http://www.ee.ic.ac.uk/hp/staff/dmb/voicebox/voicebox.html
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You can obtain a copy of the GNU General Public License from
%   ftp://prep.ai.mit.edu/pub/gnu/COPYING-2.0 or by writing to
%   Free Software Foundation, Inc.,675 Mass Ave, Cambridge, MA 02139, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

persistent BYTEORDER DATATYPE
codes={'NumFrames';  'NumChannels'};
channels={'Channel_0';...
	'Channel_1';...
	'Channel_2';...
	'Channel_3';...
	'Channel_4';...
	'Channel_5';...
	'Channel_6';...
	'Channel_7';...
	'Channel_8';...
	'Channel_9';...
	'Channel_10';...
	'Channel_11';...
	'Channel_12';...
	'Channel_13';...
	'Channel_14';...
	'Channel_15';...
	'Channel_16';...
	'Channel_17';...
	'Channel_18';...
	'Channel_19';...
	'Channel_20';...
	'Channel_21';...
	'Channel_22';...
	'Channel_23';...
	'Channel_24';...
	'Channel_25';...
	'Channel_26';...
	'Channel_27';...
	'Channel_28';...
	'Channel_29'};
if isempty(BYTEORDER) BYTEORDER='l'; end
if isempty(DATATYPE) DATATYPE='rb'; end
if nargin<1 error('Usage: [y,fs,fidx]=READEST(filename,mode,nmax,nskip)'); end
if nargin<2 mode='p';
else mode = [mode(:).' 'p'];
end
k=find((mode>='p') & (mode<='s'));
sc=mode(k(1)); 
if any(mode=='l') BYTEORDER='l';
elseif any(mode=='b') BYTEORDER='b';
end
%----------------------------------------------------------
%If the number of output arguments is not zero.....
if nargout
ffx=cell(5,1);
if ischar(filename)
   fid=fopen(filename,DATATYPE,BYTEORDER);
   if fid == -1 & isempty(findstr(filename,'.'))
      filename=[filename,'.ema'];
      fid=fopen(filename,DATATYPE,BYTEORDER);
   end
   if fid == -1 
      error(sprintf('Can''t open %s for input',filename)); 
   end
   ffx{1}=filename;
else
   if iscell(filename)
      ffx=filename;
   else
      fid=filename;
   end
end

if isempty(ffx{4});
   fseek(fid,0,-1);					
   %str=char(fread(fid,16)');
   str=fgetl(fid);
   [tok,str]=strtok(str);
   %if str(1:8) ~= 'EST_File' fclose(fid); error(sprintf('File does not begin with an EST header')); end
   if ~findstr('EST_File',tok) fclose(fid); error(sprintf('File does not begin with an EST header')); end
   ffx{3}=str;
%   hlen=str2num(str(9:15));
   hdr={};
   while 1
      str=fgetl(fid);
%NB Should really scan header for comment field to now that the comment character is ;
      if ~isempty(str) & str(1) ~= ';'
         [tok,str]=strtok(str);
         if strcmp(tok,'EST_Header_End') break; end
         hdr(end+1,1)={tok};
         [tok,str]=strtok(str);
         hdr(end,2)= {tok};
      end
   end
   hlen = ftell(fid);
%Check whether binary or ascii data
   i=find(strcmp(hdr(:,1),'DataType'));
   if ~isempty(i)
	if findstr(hdr{i,2},'ascii')
		DATATYPE = 'rt';
	elseif findstr(hdr{i,2},'binary')
                DATATYPE = 'rb';
	else
		fclose(fid); error(sprintf('DataType %s is unrecognised',hdr{i,2})); 
	end
      %DATATYPE=char('rb'+('rt'-'rb')*(hdr{i,2}(1:3)=='a'));
   end
%Check whether big-endian or little-endian binary data
   i=find(strcmp(hdr(:,1),'ByteOrder'));
   if ~isempty(i)
      bord=char('b'+('l'-'b')*(hdr{i,2}(1)=='0'));
      if bord ~= BYTEORDER & mode~='b' & mode ~='l'
         BYTEORDER=bord;
         fclose(fid);
         fid=fopen(filename,DATATYPE,BYTEORDER);
      end
   end
%Set info defaults then assign from fileheader info
%#################################################################
   info=[fid; 0; hlen; double(BYTEORDER); 0; 1; 4; 16; 1 ; 1; -1];
%#################################################################

   for j=1:2
      i=find(strcmp(hdr(:,1),codes{j}));
      if ~isempty(i)
         info(j+4)=str2num(hdr{i,2});
      end
   end

   num_chans_to_read = info(6) + 2;
%Assign channel labels 
   channel_label_cell = {};
   for j=1:30
      i=find(strcmp(hdr(:,1),channels{j}));
      if ~isempty(i)
         channel_label_cell(j)=hdr(i,2);
      end
   end
   channel_label = char(channel_label_cell);
%If number of frames is not specified in the header then calculate it
   if ~info(5)
      fseek(fid,0,1);
      info(5)=floor((ftell(fid)-info(3))/(num_chans_to_read*info(7)));
   end
   ffx{2}=hdr;
   ffx{4}=info;
   ffx{5}=channel_label;
end
info=ffx{4};
channel_label=ffx{5};
% Skip hlen bytes (header)
if nargin<4 nskip=info(2);
elseif nskip<0 nskip=info(2);
end

%ksamples is the total number of samples/channel to be read
ksamples=info(5)-nskip;
if nargin>2
   if nmax>=0
      ksamples=min(nmax,ksamples);
   end
end

if ksamples>0
   fid=info(1);
   info(2)=nskip+ksamples;
   pk=(pow2(0.5,info(8))-0.5)*pow2(1,8*info(7)-info(8));		% full-scale range
   fseek(fid,info(3)+num_chans_to_read*info(7)*nskip,-1);
%nsamples is the total number of samples to be read
   nsamples=num_chans_to_read*ksamples;

%-------------------------------------------------
%read up to 4 bytes per sample
   if info(7)<3
      if info(7)<2
         y=fread(fid,nsamples,'uchar');
         y=y-128;
      else
         y=fread(fid,nsamples,'short');
      end
   else
      if info(7)<4
         y=fread(fid,3*nsamples,'uchar');
         y=reshape(y,3,nsamples);
         y=[1 256 65536]*y-pow2(fix(pow2(y(3,:),-7)),24);
      else
         y=fread(fid,nsamples,'float');
      end
   end
%----------------------------------------------------

%Scale data if requested
   if sc ~= 'r'
      if sc=='s' 
         if info(10)>info(11)
            info(10)=min(y);
            info(11)=max(y);
         end
         pd=max(abs(info(10:11)));
         pd=pk/(pd+(pd==0));
      else pd=1;
      end
      y=pd/pk*y;
   end
   
   if num_chans_to_read>1 y = reshape(y,num_chans_to_read,ksamples).'; end
%-----------------------------------------------------
%Check the time track (track 1) to see if the sample rate is constant and store it if it is.
   t = y(:,1);
   period = t(2) - t(1);
   steps = t(2:length(t)) - t(1:length(t)-1);
%problem with noise. Fix this later.
%   if steps - ones(size(steps))*period == 0
	s = 1.0/period;
	info(9) = s;
   end
   y = y(:,3:num_chans_to_read);

%else
%   y=[];
%end

if mode~='f' fclose(fid); end
ffx{4}=info;
s=info(9);
%------------------------------------------------------
% ... else if the number of output arguments is zero then 
   if ~isempty(ffx{1}) fprintf(1,'Filename: %s\n',ffx{1}); end
   fprintf(1,'EST file type: %s\n',ffx{3});
   fprintf(1,'Duration = %ss: %d channel * %d samples @ %sHz\n',sprintsi(info(5)/info(9)),info(6),info(5),sprintsi(info(9)));
   fprintf(1,'Channel Labels are:\n');
	ffx{5}

else
   [y1,s,ffx]=readest(filename,mode,0);
   info=ffx{4};
   if ~isempty(ffx{1}) fprintf(1,'Filename: %s\n',ffx{1}); end
   fprintf(1,'EST file type: %s\n',ffx{3});
   fprintf(1,'Duration = %ss: %d channel * %d samples @ %sHz\n',sprintsi(info(5)/info(9)),info(6),info(5),sprintsi(info(9)));
end




